/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.tests.paas.test.allocatevms;

import org.glassfish.api.Param;
import org.glassfish.api.admin.AdminCommand;
import org.glassfish.api.admin.AdminCommandContext;
import org.glassfish.api.admin.CommandLock;
import org.glassfish.embeddable.CommandRunner;
import org.glassfish.virtualization.runtime.VirtualClusters;
import org.glassfish.virtualization.spi.AllocationConstraints;
import org.glassfish.virtualization.spi.AllocationPhase;
import org.glassfish.virtualization.spi.IAAS;
import org.glassfish.virtualization.spi.PhasedFuture;
import org.glassfish.virtualization.spi.TemplateInstance;
import org.glassfish.virtualization.spi.TemplateRepository;
import org.glassfish.virtualization.spi.VirtualCluster;
import org.glassfish.virtualization.spi.VirtualMachine;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.component.PerLookup;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

/**
 * @author Shalini M
 */
@Service(name = "allocate-vms")
@Scoped(PerLookup.class)
@CommandLock(CommandLock.LockType.NONE)
public class ParallelAllocation implements AdminCommand {

    @Param(name = "templates", optional = true, separator = ':')
    public List<String> templates;

    @Inject(optional = true)
    private IAAS iaas;

    @Inject(optional = true)
    private VirtualClusters virtualClusters;

    @Inject(optional = true)
    private TemplateRepository templateRepository;

    @Inject
    private CommandRunner commandRunner;

    @Param(primary = true, name = "virtualcluster")
    private String virtualClusterName;


    private ExecutorService threadPool = Executors.newCachedThreadPool();

    public void execute(AdminCommandContext context) {
        try {
            commandRunner.run("create-cluster", virtualClusterName);
            VirtualCluster vCluster = virtualClusters.byName(virtualClusterName);
            createVMs(vCluster);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    private void createVMs(final VirtualCluster vCluster) {
        for (final String templateId : templates) {
            threadPool.submit(new Runnable() {
                public void run() {
                    TemplateInstance ti = templateRepository.byName(templateId);
                    System.out.println("Creating VM using " + ti);
                    try {
                        PhasedFuture<AllocationPhase, VirtualMachine> future =
                                iaas.allocate(new AllocationConstraints(ti, vCluster), null);
                        VirtualMachine vm = future.get();
                        System.out.println("Finished allocating vm " + vm);
                    } catch (Exception ex) {
                        throw new RuntimeException(ex);
                    }
                }
            });
        }
    }

}
